<?php

namespace Drupal\bootstrap_toggle\Plugin\Field\FieldFormatter;

use Drupal\bootstrap_toggle\Plugin\Field\FieldWidget\BootstrapToggle;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\Plugin\Field\FieldFormatter\BooleanFormatter;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;

/**
 * Plugin implementation of the 'bootstrap_toggle_switch' formatter.
 *
 * @FieldFormatter(
 *   id = "bootstrap_toggle_switch",
 *   module = "bootstrap_toggle",
 *   label = @Translation("Bootstrap Toggle"),
 *   field_types = {
 *     "boolean"
 *   }
 * )
 */
class BootstrapToggleFormatter extends BooleanFormatter {

  /**
   * The BootstrapToggle widget, used to populate the Formatter.
   *
   * @var \Drupal\bootstrap_toggle\Plugin\Field\FieldWidget\BootstrapToggle
   */
  private BootstrapToggle $bootstrapToggleWidget;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings);
    // Load the BootstrapToggle Widget, for populating the Formatter methods.
    $this->bootstrapToggleWidget = new BootstrapToggle($this->pluginId, $this->pluginDefinition, $this->fieldDefinition, $this->settings, $this->thirdPartySettings);
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    // Don't Using '$this' when not in object context.
    return BootstrapToggle::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $widget = $this->bootstrapToggleWidget;
    // Empties variables, used only for instancing the BootstrapToggle widget.
    $form = [];
    $form_state = new FormState();
    $element = $widget->settingsForm($form, $form_state);

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $widget = $this->bootstrapToggleWidget;
    $summary = $widget->settingsSummary();

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];

    $widget = $this->bootstrapToggleWidget;
    // Empties variables, used only for instancing the BootstrapToggle widget.
    $form = [];
    $form_state = new FormState();

    foreach ($items as $delta => $item) {
      $element = $widget->formElement($items, $delta, $elements, $form, $form_state);
      // Remove 'value'.
      $elements[$delta] = $element['value'];
      // This a "view" display - we don't allow to change the checkbox status.
      // This is not a form - '#disabled' = TRUE and '#default_value' are
      // ignored (they don't work). As a workarround, we directly
      // inform in html, using the #attributes entry.
      $elements[$delta]['#attributes']['disabled'] = ['disabled'];
      $status = $item->value ? 'checked' : '';
      $elements[$delta]['#attributes']['checked'] = [$status];
    }
    // Add the library.
    if (isset($form['#attached'])) {
      $elements['#attached'] = $form['#attached'];
    }

    return $elements;
  }

}
